/*  
    Copyright (C) 2015 Jose A. Fernandez

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*    
    ImageJ macro for the detection of vascular, avascular and neovascular regions within the OIR model
    
    INPUT: 
	Original image: An image of the retina to be analysed needs to be opened before running the macro

    OUTPUT:
    	Result image: Original image with an overlay of colored lines for whole retina, vascular and neovascular regions
	Result Log window: Relative areas of vascular, avascular and neovascular regions as percentages of the total retina (also neovascular relative to vascular)
	Table of detected regions: ROI manager window containing all ROIs
	Table of measurements for detected regions: Different measurements can be selected in Analyze->Set Measurements...
*/

macro "OIR_Select"{
	// *** Initial parameters
	gauss_radius = 5; // Indicates the smoothness of the marked regions (default to 5)
  	ratio_oval = 3; // Indicates the size of the oval surrounding the central vasculature (default to middle size = 3, with 1 indicating the biggest and 5 indicating the smallest oval)

  	// *** User is asked to enter (or accept) these values...
  	Dialog.create("Enter the following parameters:");
  	Dialog.addNumber("Filter radius [3-20]:", gauss_radius);
  	Dialog.addNumber("Oval size [1-5]:", ratio_oval);
  	Dialog.show();
  	gauss_radius = Dialog.getNumber(); // between 3 and 20
  	ratio_oval = Dialog.getNumber(); // between 1 and 5

	// *** Filters for different thresholdings
	external_filter = "Huang dark"; // Filter for the detection of the total retina (affected by gauss_radius)
	vascular_filter = "Otsu dark"; // Filter for the detection of the vascular regions (affected by gauss_radius)
	ivascular_filter = "Otsu dark"; // Filter for the detection of the central vascular regions
	damaged_filter = "Otsu dark"; // Filter for the detection of the neovascular regions

	// *** Extra parameters for processing and drawing (defaults are good enough)
	damaged_erode = 0; // Provides some control over the detection of the neovascular regions (not needed)
	damaged_removed = 30; // Minimum size of the neovascular regions to be detected (see User manual for an explanation)
	line_width = 1; // Width of the line to draw regions (this can also be changed using the ImageJ tools)
	flag_external = 0; // If Flag=0, the whole image area is used for percentage calculation instead of total retina area (see User manual for an explanation)

	// *** Pre-format original image (e.g. background colors, selection line colors, etc)
	run("Options...", "iterations=1 black pad edm=Overwrite count=1"); 
	run("Colors...", "foreground=white background=black selection=yellow"); 
		
	// *** Get data from original image (e.g. ID value, dimensions, etc)	
	original_ID = getImageID();
	getDimensions(width, height, channels, slices, frames);
	external_size = (width * height) / 5; // the retina has to fill at least a fourth the screen (was 10000)

	// *** Create several images for posterior processing and get handles (IDs) to each of them 
	//	(1) Image to mark external border of whole retina
	run("Duplicate...", "title=[external.bmp]");
	external_ID = getImageID();
	run("8-bit");
	run("Enhance Contrast","saturation=0.35");
	run("Apply LUT");
	//	(2) Image to mark central vasculature
	run("Duplicate...", "title=[int_vasc.bmp]");
	int_vasc_ID = getImageID();
	//	(3) Image to mark neovascular regions
	run("Duplicate...", "title=[damaged.bmp]");
	damaged_ID = getImageID();
	// *** Process initial image (apply Gaussian filter)
	selectImage(external_ID);
	run("Gaussian Blur...", "sigma="+gauss_radius); 
	//	(4) Image to mark peripheral vascular regions
	run("Duplicate...", "title=[vascular.bmp]");
	vascular_ID = getImageID();

	// *** Process image (1) to select border of the full retina
	selectImage(original_ID);
	thr1_ID = get_thr(external_ID, external_filter, "external retina");
	run("Fill Holes");
	selectImage(int_vasc_ID);
	imageCalculator("AND create", "external.bmp", "int_vasc.bmp");
	int_vasc_ID2 = getImageID(); selectImage(int_vasc_ID); close();
	
	// *** Get oval dimensions (to analyse center of retina separately)
	selectImage(thr1_ID);
	run("Analyze Particles...", "size="+external_size+"-Infinity circularity=0.00-1.00 show=Nothing add");
	run("Set Measurements...", "area center bounding display redirect=None decimal=3");
	roiManager("select all"); roiManager("measure");
	x_oval = getResult("XM", 0);
	y_oval = getResult("YM", 0);
	width_oval = getResult("Width", 0);
	height_oval = getResult("Height", 0);
	selectImage(thr1_ID); close();
	roiManager("Set Color", "blue");
	roiManager("Set Line Width", line_width);
	if ((roiManager("count") == 0) || (roiManager("count") > 1)){
		roiManager("reset");
		flag_external = 0;
	}
	
	// *** Process vascular regions (both image (2) for peripheral and image (4) for central)
	//	(A) Peripheral vascular regions (near the borders)
	roiManager("Deselect"); selectImage(original_ID);
	thr2_ID = get_thr(vascular_ID, vascular_filter, "vascular");
	makeOval(x_oval-(width_oval/(ratio_oval*2)), y_oval-(height_oval/(ratio_oval*2)), width_oval/ratio_oval, height_oval/ratio_oval);
	roiManager("Add"); oval_ROI = roiManager("count") - 1;
	roiManager("Select", oval_ROI); setForegroundColor(0, 0, 0); roiManager("Fill");
	run("Analyze Particles...", "size=10000-Infinity circularity=0.00-1.00 show=Nothing add");
	roiManager("Select", oval_ROI); roiManager("Delete");
	nvascular = roiManager("count") - 1; 
	for (i = 0; i < nvascular; i++){
		roiManager("Select", i + 1);
		roiManager("Set Color", "red");
		roiManager("Set Line Width", line_width);
	}
	//	(B) Central vascular regions (around the center)
	selectImage(int_vasc_ID2); int_title = getTitle();
	imageCalculator("Substract create", int_title, "vascular.bmp"); 
	result = getImageID();
	selectImage(int_vasc_ID2); close(); 
	selectImage(thr2_ID); close();
	selectImage(original_ID);
	result = get_thr(result, ivascular_filter, "internal vascular");
	run("Analyze Particles...", "size=100-Infinity circularity=50.00-1.00 show=Nothing add");
	selectImage(result); close();
	nivascular = roiManager("count") - nvascular - 1; 
	for (i = 0; i < nivascular; i++){
		roiManager("Select", i + nvascular + 1);
		roiManager("Set Color", "red");
		roiManager("Set Line Width", line_width);
	}
	
	// *** Process image (3) for neovascular regions
	selectImage(original_ID);
	thr_dam_ID = get_thr(damaged_ID, damaged_filter, "neovascular");
	selectImage(thr_dam_ID);
	for (i=0; i<damaged_erode;i++) run("Erode"); // Not needed in most cases
	thr_ID = getImageID();
	makeOval(x_oval-(width_oval/(ratio_oval*2)), y_oval-(height_oval/(ratio_oval*2)), width_oval/ratio_oval, height_oval/ratio_oval);
	roiManager("Add"); oval_ROI = roiManager("count") - 1;
	roiManager("Select", oval_ROI); setForegroundColor(0, 0, 0); roiManager("Fill");
	run("Analyze Particles...", "size="+damaged_removed+"-Infinity circularity=50.00-1.00 show=Nothing add");
	selectImage(thr_ID); close();
	roiManager("Select", oval_ROI); roiManager("Delete");
	ndamaged = roiManager("count") - nivascular - nvascular - 1; // was -1; removing vascular and external

	// *** Measure all relative areas
	if (is("locked")) run("Reset...", "reset=[Locked Image]");
	roiManager("Select All"); roiManager("Measure");
	if (flag_external)
		ext_area = getResult("Area",1); // if Flag=1, use area of whole retina for relative measurements
	else
		ext_area = height * width; // if Flag=0, use area of whole image
	vasc_area = 0;
	for (i = 0; i < nvascular; i++){
		vasc_area = vasc_area + getResult("Area",i+2); 
	}
	ivasc_area = 0;
	for (i = 0; i < nivascular; i++){
		ivasc_area = ivasc_area + getResult("Area",i+nvascular+2); 
	}
	dam_area = 0;
	for (i = 0; i < ndamaged; i++){
		dam_area = dam_area + getResult("Area",i+nvascular+nivascular+2); 
	}
	total_vasc_area = vasc_area + ivasc_area;
	
	// *** Print output relative areas to log window
	print("\\Clear"); 
	print("RELATIVE AREAS\t\t\t");
	print("1. Vascular and avascular:\t\t\t");
	print("     Avascular ((total-vascular)*100/total): " + (ext_area-total_vasc_area)*100/ext_area + " %\t\t\t");
	print("     Vascular (vascular*100/total): " + total_vasc_area*100/ext_area + " %\t\t\t");
	print("\t\t\t");
	print("2. Avascular, normal and neovascular:\t\t\t");
	print("     Avascular ((total-vascular)*100/total): " + (ext_area-total_vasc_area)*100/ext_area + " %\t\t\t");
	print("     Normal vascular ((vascular-neovascular)*100/total): " + (total_vasc_area-dam_area)*100/ext_area + " %\t\t\t");
	print("     Neovascular (neovascular*100/total): " + dam_area*100/ext_area + " %\t\t\t");
	print("\t\t\t");
	print("3. Neovascular (neovascular*100/vascular): " + dam_area*100/total_vasc_area + " %\t\t\t");
	roiManager("Show all without labels"); // show all detected regions (with labels OFF)
}

// Function to convert original image to thresholded one
// INPUT: 
//	orig_ID: Original image ID (directly converted, so it doesn't need to be closed)
//	type: String containing the type of thresholding method (e.g. "Otsu dark")
//	detection: String indicating the region set for detection
// OUTPUT: ID of thresholded (black-and-white) image
function get_thr(orig_ID, type, detection){
	selectImage(orig_ID);
	setAutoThreshold(type);
	run("Threshold...");
	getThreshold(lower,upper);
	setThreshold(lower,upper);
	waitForUser("Please, set threshold for " + detection + " detection...");
	setOption("BlackBackground", true);
	run("Convert to Mask");
	if (is("Inverting LUT")) run("Invert LUT");
	return (getImageID());
}
